// In order to *not* need this ignore, consider extracting the "web" version
// of your plugin as a separate package, instead of inlining it in the same
// package as the core of your plugin.
// ignore: avoid_web_libraries_in_flutter

import 'dart:async';

import 'package:amap_flutter_location/amap_location_plugin_platform_interface.dart';
import 'package:amap_flutter_location/web/web_location_imp.dart';
import 'package:flutter_web_plugins/flutter_web_plugins.dart';

import 'amap_location_option.dart';
import 'package:location/location.dart';

/// A web implementation of the TestWebPluginPlatform of the TestWebPlugin plugin.
class AmapLocationPluginWeb extends AmapLocationPluginPlatform {
  /// Constructs a AmapLocationPluginWeb
  AmapLocationPluginWeb();

  String? _pluginKey;
  Map<String, WebLocationImp> _webLocationImpMap = {};
  StreamController<Map<String, Object?>>? _receiveStream;
  StreamSubscription<Map<String, Object?>>? _subscription;

  static void registerWith(Registrar registrar) {
    AmapLocationPluginPlatform.instance = AmapLocationPluginWeb();
  }

  /// 初始化key
  @override
  void initKey() {
    _pluginKey = DateTime.now().millisecondsSinceEpoch.toString();
  }

  /// 隐私权政策弹窗展示告知用户
  @override
  void updatePrivacyShow(bool hasContains, bool hasShow) {
    // 暂不需要
  }

  /// 隐私权政策取得用户同意
  @override
  void updatePrivacyAgree(bool hasAgree) {
    // 暂不需要
  }

  /// 设置apiKey
  @override
  void setApiKey(String androidKey, String iosKey, String webKey) {
    // 暂不需要
  }

  /// 设置定位参数
  @override
  void setLocationOption(AMapLocationOption locationOption) {
    Map option = locationOption.getOptionsMap();
    option['pluginKey'] = _pluginKey;
    _getWebLocationImp(option)?.setLocationOption(option);
  }

  /// 开始定位
  @override
  void startLocation() {
    _getWebLocationImp({"pluginKey": _pluginKey})?.startLocation();
  }

  /// 停止定位
  @override
  void stopLocation() {
    _getWebLocationImp({"pluginKey": _pluginKey})?.stopLocation();
  }

  /// 定位结果回调
  @override
  Stream<Map<String, Object?>> onLocationChanged() {
    if (_receiveStream == null) {
      _receiveStream = StreamController();
      WebLocationImp? webLocationImp =
          _getWebLocationImp({"pluginKey": _pluginKey});
      _subscription = webLocationImp?.onLocationChanged()?.listen(
          (Map<String, Object?> event) {
        if (event['pluginKey'] == _pluginKey) {
          Map<String, Object?> newEvent = Map<String, Object?>.of(event);
          newEvent.remove('pluginKey');
          _receiveStream?.add(newEvent);
        }
      }, onError: (Object error) {
        print('onLocationChanged error: $error');
        _receiveStream?.addError(error);
      }, onDone: () {
        print('onLocationChanged done');
        _receiveStream?.close();
      });
    }
    return _receiveStream!.stream;
  }

  /// 销毁定位
  @override
  void destroy() {
    WebLocationImp? webLocationImp =
        _getWebLocationImp({"pluginKey": _pluginKey});
    webLocationImp?.destroy();
    _webLocationImpMap.remove(webLocationImp);
    if (_subscription != null) {
      _receiveStream?.close();
      _subscription?.cancel();
      _receiveStream = null;
      _subscription = null;
    }
  }

  /// 获取web定位实现
  WebLocationImp? _getWebLocationImp(Map? argsMap) {
    String? pluginKey = argsMap?["pluginKey"];
    if (pluginKey == null || pluginKey.isEmpty) return null;
    if (!_webLocationImpMap.containsKey(pluginKey)) {
      WebLocationImp webLocationImp = WebLocationImp(pluginKey: pluginKey);
      _webLocationImpMap[pluginKey] = webLocationImp;
    }
    return _webLocationImpMap[pluginKey];
  }
}
