import 'dart:async';

import 'package:location/location.dart';

/// web端定位实现
class WebLocationImp {
  Location? _location;

  String? pluginKey;

  /// StreamController
  StreamController<Map<String, Object?>>? streamController =
      StreamController<Map<String, Object?>>();

  WebLocationImp({required this.pluginKey}) {
    _location ??= Location();
  }

  /// 开启定位
  void startLocation() async {
    try {
      // 检测是否开启定位服务
      bool? _serviceEnabled = await _location?.serviceEnabled();
      if (_serviceEnabled == false) {
        _serviceEnabled = await _location?.requestService();
        if (_serviceEnabled == false) {
          assert(false, "定位服务未开启");
          return;
        }
      }
      // 检测是否开启定位权限
      PermissionStatus? _permissionGranted = await _location?.hasPermission();
      if (_permissionGranted == PermissionStatus.denied) {
        _permissionGranted = await _location?.requestPermission();
        if (_permissionGranted != PermissionStatus.granted) {
          assert(false, "定位权限未开启");
          return;
        }
      }
      // 获取定位信息
      LocationData? locationData = await _location?.getLocation();
      var map = {
        "latitude": locationData?.latitude,
        "longitude": locationData?.longitude,
        "pluginKey": pluginKey,
      };
      streamController?.add(map);
    } catch (e) {
      print("定位获取异常：$e");
      assert(false, "定位获取异常：$e");
    }
  }

  /// 定位回调
  Stream<Map<String, Object?>>? onLocationChanged() {
    return streamController?.stream;
  }

  /// 设置定位参数
  void setLocationOption(Map optionMap) {}

  /// 停止定位
  void stopLocation() {}

  /// 销毁定位
  void destroy() {
    _location = null;
    streamController?.close();
    streamController = null;
  }
}
